"""The s commands module.

These commands are used in the following models:
TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - S<x>_CH<x>:BANdwidth:ACTUal?
    - S<x>_CH<x>:CLIPping?
    - S<x>_CH<x>:COUPling {AC|DC|DCREJ}
    - S<x>_CH<x>:COUPling?
    - S<x>_CH<x>:DESKew <NR3>
    - S<x>_CH<x>:DESKew?
    - S<x>_CH<x>:LABel:COLor <QString>
    - S<x>_CH<x>:LABel:FONT:BOLD {ON|OFF|<NR1>}
    - S<x>_CH<x>:LABel:FONT:ITALic {ON|OFF|<NR1>}
    - S<x>_CH<x>:LABel:FONT:SIZE <NR1>
    - S<x>_CH<x>:LABel:FONT:TYPE <QString>
    - S<x>_CH<x>:LABel:FONT:UNDERline {ON|OFF|<NR1>}
    - S<x>_CH<x>:LABel:NAMe <QString>
    - S<x>_CH<x>:LABel:NAMe?
    - S<x>_CH<x>:LABel:XPOS <NR3>
    - S<x>_CH<x>:LABel:XPOS?
    - S<x>_CH<x>:LABel:YPOS <NR3>
    - S<x>_CH<x>:LABel:YPOS?
    - S<x>_CH<x>:OFFSet <NR3>
    - S<x>_CH<x>:OFFSet?
    - S<x>_CH<x>:POSition <NR1>
    - S<x>_CH<x>:SCAle <NR3>
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING

from ..helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedChannel,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class SItemChannelScale(SCPICmdWrite):
    """The ``S<x>_CH<x>:SCAle`` command.

    Description:
        - This command sets or returns the vertical scale for the specified remote scope analog
          channel. The remote scope and the channel is specified by x.

    Usage:
        - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:SCAle value`` command.

    SCPI Syntax:
        ```
        - S<x>_CH<x>:SCAle <NR3>
        ```

    Info:
        - ``<NR3>`` is the vertical scale for the specified analog channel.
    """


class SItemChannelPosition(SCPICmdWrite):
    """The ``S<x>_CH<x>:POSition`` command.

    Description:
        - This command sets or queries the vertical position for the specified analog channel. The
          remote scope and channel is specified by x.

    Usage:
        - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:POSition value`` command.

    SCPI Syntax:
        ```
        - S<x>_CH<x>:POSition <NR1>
        ```

    Info:
        - ``<NR1>`` is the vertical position for the specified analog channel.
    """


class SItemChannelOffset(SCPICmdWrite, SCPICmdRead):
    """The ``S<x>_CH<x>:OFFSet`` command.

    Description:
        - This command sets or queries the vertical offset for the specified remote scope analog
          channel. The remote scope and channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``S<x>_CH<x>:OFFSet?`` query.
        - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:OFFSet?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:OFFSet value`` command.

    SCPI Syntax:
        ```
        - S<x>_CH<x>:OFFSet <NR3>
        - S<x>_CH<x>:OFFSet?
        ```

    Info:
        - ``<NR3>`` is the offset value for the specified remote scope analog channel.
    """


class SItemChannelLabelYpos(SCPICmdWrite, SCPICmdRead):
    """The ``S<x>_CH<x>:LABel:YPOS`` command.

    Description:
        - This command sets or queries the Y-position of the specified remote scope channel. The
          remote scope and channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``S<x>_CH<x>:LABel:YPOS?`` query.
        - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:LABel:YPOS?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:LABel:YPOS value`` command.

    SCPI Syntax:
        ```
        - S<x>_CH<x>:LABel:YPOS <NR3>
        - S<x>_CH<x>:LABel:YPOS?
        ```

    Info:
        - ``<NR3>`` is the location (in pixels) where the waveform label for the selected channel is
          displayed, relative to the baseline of the waveform. Positive values are above the
          baseline and negative values are below.
    """


class SItemChannelLabelXpos(SCPICmdWrite, SCPICmdRead):
    """The ``S<x>_CH<x>:LABel:XPOS`` command.

    Description:
        - This command sets or queries the X-position of the specified remote scope channel. The
          remote scope and channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``S<x>_CH<x>:LABel:XPOS?`` query.
        - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:LABel:XPOS?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:LABel:XPOS value`` command.

    SCPI Syntax:
        ```
        - S<x>_CH<x>:LABel:XPOS <NR3>
        - S<x>_CH<x>:LABel:XPOS?
        ```

    Info:
        - ``<NR3>`` is the location (in pixels) where the waveform label for the selected channel is
          displayed, relative to the left edge of the screen.
    """


class SItemChannelLabelName(SCPICmdWrite, SCPICmdRead):
    """The ``S<x>_CH<x>:LABel:NAMe`` command.

    Description:
        - This command sets or queries the label attached to the displayed waveform for the
          specified remote scope channel. The remote scope and channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``S<x>_CH<x>:LABel:NAMe?`` query.
        - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:LABel:NAMe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:LABel:NAMe value`` command.

    SCPI Syntax:
        ```
        - S<x>_CH<x>:LABel:NAMe <QString>
        - S<x>_CH<x>:LABel:NAMe?
        ```

    Info:
        - ``<QString>`` is an alphanumeric character string, ranging from 1 through 32 characters in
          length.
    """

    _WRAP_ARG_WITH_QUOTES = True


class SItemChannelLabelFontUnderline(SCPICmdWrite):
    """The ``S<x>_CH<x>:LABel:FONT:UNDERline`` command.

    Description:
        - This command sets or queries the underline state of the specified channel label. The
          channel is specified by x.

    Usage:
        - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:LABel:FONT:UNDERline value``
          command.

    SCPI Syntax:
        ```
        - S<x>_CH<x>:LABel:FONT:UNDERline {ON|OFF|<NR1>}
        ```

    Info:
        - ``OFF`` argument turns off underlined font.
        - ``ON`` argument turns on underlined font.
        - ``<NR1>`` = 0 turns off underlined font; any other value turns on underlined font.
    """


class SItemChannelLabelFontType(SCPICmdWrite):
    """The ``S<x>_CH<x>:LABel:FONT:TYPE`` command.

    Description:
        - This command sets or queries the font type of the specified remote scope channel label,
          such as Arial or Times New Roman. The remote scope and channel is specified by x.

    Usage:
        - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:LABel:FONT:TYPE value``
          command.

    SCPI Syntax:
        ```
        - S<x>_CH<x>:LABel:FONT:TYPE <QString>
        ```

    Info:
        - ``<QString>`` is the specified font type.
    """

    _WRAP_ARG_WITH_QUOTES = True


class SItemChannelLabelFontSize(SCPICmdWrite):
    """The ``S<x>_CH<x>:LABel:FONT:SIZE`` command.

    Description:
        - This command sets or queries the font size of the specified channel label. The remote
          scope and channel is specified by x.

    Usage:
        - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:LABel:FONT:SIZE value``
          command.

    SCPI Syntax:
        ```
        - S<x>_CH<x>:LABel:FONT:SIZE <NR1>
        ```

    Info:
        - ``<NR1>`` is the font size.
    """


class SItemChannelLabelFontItalic(SCPICmdWrite):
    """The ``S<x>_CH<x>:LABel:FONT:ITALic`` command.

    Description:
        - This command sets or queries the italic state of the specified remote scope channel label.
          The remote scope and channel is specified by x.

    Usage:
        - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:LABel:FONT:ITALic value``
          command.

    SCPI Syntax:
        ```
        - S<x>_CH<x>:LABel:FONT:ITALic {ON|OFF|<NR1>}
        ```

    Info:
        - ``OFF`` argument turns off italic font.
        - ``ON`` argument turns on italic font.
        - ``<NR1>`` = 0 turns off italic font; any other value turns on italic font.
    """


class SItemChannelLabelFontBold(SCPICmdWrite):
    """The ``S<x>_CH<x>:LABel:FONT:BOLD`` command.

    Description:
        - This command sets or queries the bold state of the specified remote scope channel label.
          The remote scope and channel is specified by x.

    Usage:
        - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:LABel:FONT:BOLD value``
          command.

    SCPI Syntax:
        ```
        - S<x>_CH<x>:LABel:FONT:BOLD {ON|OFF|<NR1>}
        ```

    Info:
        - ``OFF`` argument turns off bold font.
        - ``ON`` argument turns on bold font.
        - ``<NR1>`` = 0 turns off bold font; any other value turns on bold font.
    """


class SItemChannelLabelFont(SCPICmdRead):
    """The ``S<x>_CH<x>:LABel:FONT`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``S<x>_CH<x>:LABel:FONT?`` query.
        - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:LABel:FONT?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.bold``: The ``S<x>_CH<x>:LABel:FONT:BOLD`` command.
        - ``.italic``: The ``S<x>_CH<x>:LABel:FONT:ITALic`` command.
        - ``.size``: The ``S<x>_CH<x>:LABel:FONT:SIZE`` command.
        - ``.type``: The ``S<x>_CH<x>:LABel:FONT:TYPE`` command.
        - ``.underline``: The ``S<x>_CH<x>:LABel:FONT:UNDERline`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._bold = SItemChannelLabelFontBold(device, f"{self._cmd_syntax}:BOLD")
        self._italic = SItemChannelLabelFontItalic(device, f"{self._cmd_syntax}:ITALic")
        self._size = SItemChannelLabelFontSize(device, f"{self._cmd_syntax}:SIZE")
        self._type = SItemChannelLabelFontType(device, f"{self._cmd_syntax}:TYPE")
        self._underline = SItemChannelLabelFontUnderline(device, f"{self._cmd_syntax}:UNDERline")

    @property
    def bold(self) -> SItemChannelLabelFontBold:
        """Return the ``S<x>_CH<x>:LABel:FONT:BOLD`` command.

        Description:
            - This command sets or queries the bold state of the specified remote scope channel
              label. The remote scope and channel is specified by x.

        Usage:
            - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:LABel:FONT:BOLD value``
              command.

        SCPI Syntax:
            ```
            - S<x>_CH<x>:LABel:FONT:BOLD {ON|OFF|<NR1>}
            ```

        Info:
            - ``OFF`` argument turns off bold font.
            - ``ON`` argument turns on bold font.
            - ``<NR1>`` = 0 turns off bold font; any other value turns on bold font.
        """
        return self._bold

    @property
    def italic(self) -> SItemChannelLabelFontItalic:
        """Return the ``S<x>_CH<x>:LABel:FONT:ITALic`` command.

        Description:
            - This command sets or queries the italic state of the specified remote scope channel
              label. The remote scope and channel is specified by x.

        Usage:
            - Using the ``.write(value)`` method will send the
              ``S<x>_CH<x>:LABel:FONT:ITALic value`` command.

        SCPI Syntax:
            ```
            - S<x>_CH<x>:LABel:FONT:ITALic {ON|OFF|<NR1>}
            ```

        Info:
            - ``OFF`` argument turns off italic font.
            - ``ON`` argument turns on italic font.
            - ``<NR1>`` = 0 turns off italic font; any other value turns on italic font.
        """
        return self._italic

    @property
    def size(self) -> SItemChannelLabelFontSize:
        """Return the ``S<x>_CH<x>:LABel:FONT:SIZE`` command.

        Description:
            - This command sets or queries the font size of the specified channel label. The remote
              scope and channel is specified by x.

        Usage:
            - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:LABel:FONT:SIZE value``
              command.

        SCPI Syntax:
            ```
            - S<x>_CH<x>:LABel:FONT:SIZE <NR1>
            ```

        Info:
            - ``<NR1>`` is the font size.
        """
        return self._size

    @property
    def type(self) -> SItemChannelLabelFontType:
        """Return the ``S<x>_CH<x>:LABel:FONT:TYPE`` command.

        Description:
            - This command sets or queries the font type of the specified remote scope channel
              label, such as Arial or Times New Roman. The remote scope and channel is specified by
              x.

        Usage:
            - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:LABel:FONT:TYPE value``
              command.

        SCPI Syntax:
            ```
            - S<x>_CH<x>:LABel:FONT:TYPE <QString>
            ```

        Info:
            - ``<QString>`` is the specified font type.
        """
        return self._type

    @property
    def underline(self) -> SItemChannelLabelFontUnderline:
        """Return the ``S<x>_CH<x>:LABel:FONT:UNDERline`` command.

        Description:
            - This command sets or queries the underline state of the specified channel label. The
              channel is specified by x.

        Usage:
            - Using the ``.write(value)`` method will send the
              ``S<x>_CH<x>:LABel:FONT:UNDERline value`` command.

        SCPI Syntax:
            ```
            - S<x>_CH<x>:LABel:FONT:UNDERline {ON|OFF|<NR1>}
            ```

        Info:
            - ``OFF`` argument turns off underlined font.
            - ``ON`` argument turns on underlined font.
            - ``<NR1>`` = 0 turns off underlined font; any other value turns on underlined font.
        """
        return self._underline


class SItemChannelLabelColor(SCPICmdWrite):
    """The ``S<x>_CH<x>:LABel:COLor`` command.

    Description:
        - This command sets or queries the color of the specified remote scope channel label. The
          remote scope and channel is specified by x.

    Usage:
        - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:LABel:COLor value`` command.

    SCPI Syntax:
        ```
        - S<x>_CH<x>:LABel:COLor <QString>
        ```

    Info:
        - ``<QString>`` is the label color. To return the color to the default color, send an empty
          string as in this example: ``S1_CH5:LABEL:COLOR`` ''.
    """

    _WRAP_ARG_WITH_QUOTES = True


class SItemChannelLabel(SCPICmdRead):
    """The ``S<x>_CH<x>:LABel`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``S<x>_CH<x>:LABel?`` query.
        - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:LABel?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.color``: The ``S<x>_CH<x>:LABel:COLor`` command.
        - ``.font``: The ``S<x>_CH<x>:LABel:FONT`` command tree.
        - ``.name``: The ``S<x>_CH<x>:LABel:NAMe`` command.
        - ``.xpos``: The ``S<x>_CH<x>:LABel:XPOS`` command.
        - ``.ypos``: The ``S<x>_CH<x>:LABel:YPOS`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._color = SItemChannelLabelColor(device, f"{self._cmd_syntax}:COLor")
        self._font = SItemChannelLabelFont(device, f"{self._cmd_syntax}:FONT")
        self._name = SItemChannelLabelName(device, f"{self._cmd_syntax}:NAMe")
        self._xpos = SItemChannelLabelXpos(device, f"{self._cmd_syntax}:XPOS")
        self._ypos = SItemChannelLabelYpos(device, f"{self._cmd_syntax}:YPOS")

    @property
    def color(self) -> SItemChannelLabelColor:
        """Return the ``S<x>_CH<x>:LABel:COLor`` command.

        Description:
            - This command sets or queries the color of the specified remote scope channel label.
              The remote scope and channel is specified by x.

        Usage:
            - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:LABel:COLor value``
              command.

        SCPI Syntax:
            ```
            - S<x>_CH<x>:LABel:COLor <QString>
            ```

        Info:
            - ``<QString>`` is the label color. To return the color to the default color, send an
              empty string as in this example: ``S1_CH5:LABEL:COLOR`` ''.
        """
        return self._color

    @property
    def font(self) -> SItemChannelLabelFont:
        """Return the ``S<x>_CH<x>:LABel:FONT`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``S<x>_CH<x>:LABel:FONT?`` query.
            - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:LABel:FONT?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.bold``: The ``S<x>_CH<x>:LABel:FONT:BOLD`` command.
            - ``.italic``: The ``S<x>_CH<x>:LABel:FONT:ITALic`` command.
            - ``.size``: The ``S<x>_CH<x>:LABel:FONT:SIZE`` command.
            - ``.type``: The ``S<x>_CH<x>:LABel:FONT:TYPE`` command.
            - ``.underline``: The ``S<x>_CH<x>:LABel:FONT:UNDERline`` command.
        """
        return self._font

    @property
    def name(self) -> SItemChannelLabelName:
        """Return the ``S<x>_CH<x>:LABel:NAMe`` command.

        Description:
            - This command sets or queries the label attached to the displayed waveform for the
              specified remote scope channel. The remote scope and channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``S<x>_CH<x>:LABel:NAMe?`` query.
            - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:LABel:NAMe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:LABel:NAMe value``
              command.

        SCPI Syntax:
            ```
            - S<x>_CH<x>:LABel:NAMe <QString>
            - S<x>_CH<x>:LABel:NAMe?
            ```

        Info:
            - ``<QString>`` is an alphanumeric character string, ranging from 1 through 32
              characters in length.
        """
        return self._name

    @property
    def xpos(self) -> SItemChannelLabelXpos:
        """Return the ``S<x>_CH<x>:LABel:XPOS`` command.

        Description:
            - This command sets or queries the X-position of the specified remote scope channel. The
              remote scope and channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``S<x>_CH<x>:LABel:XPOS?`` query.
            - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:LABel:XPOS?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:LABel:XPOS value``
              command.

        SCPI Syntax:
            ```
            - S<x>_CH<x>:LABel:XPOS <NR3>
            - S<x>_CH<x>:LABel:XPOS?
            ```

        Info:
            - ``<NR3>`` is the location (in pixels) where the waveform label for the selected
              channel is displayed, relative to the left edge of the screen.
        """
        return self._xpos

    @property
    def ypos(self) -> SItemChannelLabelYpos:
        """Return the ``S<x>_CH<x>:LABel:YPOS`` command.

        Description:
            - This command sets or queries the Y-position of the specified remote scope channel. The
              remote scope and channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``S<x>_CH<x>:LABel:YPOS?`` query.
            - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:LABel:YPOS?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:LABel:YPOS value``
              command.

        SCPI Syntax:
            ```
            - S<x>_CH<x>:LABel:YPOS <NR3>
            - S<x>_CH<x>:LABel:YPOS?
            ```

        Info:
            - ``<NR3>`` is the location (in pixels) where the waveform label for the selected
              channel is displayed, relative to the baseline of the waveform. Positive values are
              above the baseline and negative values are below.
        """
        return self._ypos


class SItemChannelDeskew(SCPICmdWrite, SCPICmdRead):
    """The ``S<x>_CH<x>:DESKew`` command.

    Description:
        - This command sets or queries the horizontal deskew time for the specified channel. The
          remote scope and channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``S<x>_CH<x>:DESKew?`` query.
        - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:DESKew?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:DESKew value`` command.

    SCPI Syntax:
        ```
        - S<x>_CH<x>:DESKew <NR3>
        - S<x>_CH<x>:DESKew?
        ```

    Info:
        - ``<NR3>`` is the deskew time for this channel, ranging from -125 ns to +125 ns with a
          resolution of 40 ps. Out-of-range values are clipped.
    """


class SItemChannelCoupling(SCPICmdWrite, SCPICmdRead):
    """The ``S<x>_CH<x>:COUPling`` command.

    Description:
        - This command sets or queries the input coupling setting for the specified remote scope
          analog channel. The remote scope and the channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``S<x>_CH<x>:COUPling?`` query.
        - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:COUPling?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:COUPling value`` command.

    SCPI Syntax:
        ```
        - S<x>_CH<x>:COUPling {AC|DC|DCREJ}
        - S<x>_CH<x>:COUPling?
        ```

    Info:
        - ``AC`` sets the specified channel to AC coupling.
        - ``DC`` sets the specified channel to DC coupling.
        - ``DCREJect`` sets DC Reject coupling when probes are attached that support that feature.
    """


class SItemChannelClipping(SCPICmdRead):
    """The ``S<x>_CH<x>:CLIPping`` command.

    Description:
        - Queries whether the specified remote scope channel's input signal is clipping (exceeding)
          the channel A/D converter range. The remote scope and channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``S<x>_CH<x>:CLIPping?`` query.
        - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:CLIPping?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - S<x>_CH<x>:CLIPping?
        ```
    """


class SItemChannelBandwidthActual(SCPICmdRead):
    """The ``S<x>_CH<x>:BANdwidth:ACTUal`` command.

    Description:
        - Queries whether the actual bandwidth of the specified remote scope channel. The remote
          scope and channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``S<x>_CH<x>:BANdwidth:ACTUal?`` query.
        - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:BANdwidth:ACTUal?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - S<x>_CH<x>:BANdwidth:ACTUal?
        ```
    """


class SItemChannelBandwidth(SCPICmdRead):
    """The ``S<x>_CH<x>:BANdwidth`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``S<x>_CH<x>:BANdwidth?`` query.
        - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:BANdwidth?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.actual``: The ``S<x>_CH<x>:BANdwidth:ACTUal`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._actual = SItemChannelBandwidthActual(device, f"{self._cmd_syntax}:ACTUal")

    @property
    def actual(self) -> SItemChannelBandwidthActual:
        """Return the ``S<x>_CH<x>:BANdwidth:ACTUal`` command.

        Description:
            - Queries whether the actual bandwidth of the specified remote scope channel. The remote
              scope and channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``S<x>_CH<x>:BANdwidth:ACTUal?`` query.
            - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:BANdwidth:ACTUal?``
              query and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - S<x>_CH<x>:BANdwidth:ACTUal?
            ```
        """
        return self._actual


#  pylint: disable=too-many-instance-attributes
class SItemChannel(ValidatedChannel, SCPICmdRead):
    """The ``S<x>_CH<x>`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``S<x>_CH<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.bandwidth``: The ``S<x>_CH<x>:BANdwidth`` command tree.
        - ``.clipping``: The ``S<x>_CH<x>:CLIPping`` command.
        - ``.coupling``: The ``S<x>_CH<x>:COUPling`` command.
        - ``.deskew``: The ``S<x>_CH<x>:DESKew`` command.
        - ``.label``: The ``S<x>_CH<x>:LABel`` command tree.
        - ``.offset``: The ``S<x>_CH<x>:OFFSet`` command.
        - ``.position``: The ``S<x>_CH<x>:POSition`` command.
        - ``.scale``: The ``S<x>_CH<x>:SCAle`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._bandwidth = SItemChannelBandwidth(device, f"{self._cmd_syntax}:BANdwidth")
        self._clipping = SItemChannelClipping(device, f"{self._cmd_syntax}:CLIPping")
        self._coupling = SItemChannelCoupling(device, f"{self._cmd_syntax}:COUPling")
        self._deskew = SItemChannelDeskew(device, f"{self._cmd_syntax}:DESKew")
        self._label = SItemChannelLabel(device, f"{self._cmd_syntax}:LABel")
        self._offset = SItemChannelOffset(device, f"{self._cmd_syntax}:OFFSet")
        self._position = SItemChannelPosition(device, f"{self._cmd_syntax}:POSition")
        self._scale = SItemChannelScale(device, f"{self._cmd_syntax}:SCAle")

    @property
    def bandwidth(self) -> SItemChannelBandwidth:
        """Return the ``S<x>_CH<x>:BANdwidth`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``S<x>_CH<x>:BANdwidth?`` query.
            - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:BANdwidth?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.actual``: The ``S<x>_CH<x>:BANdwidth:ACTUal`` command.
        """
        return self._bandwidth

    @property
    def clipping(self) -> SItemChannelClipping:
        """Return the ``S<x>_CH<x>:CLIPping`` command.

        Description:
            - Queries whether the specified remote scope channel's input signal is clipping
              (exceeding) the channel A/D converter range. The remote scope and channel is specified
              by x.

        Usage:
            - Using the ``.query()`` method will send the ``S<x>_CH<x>:CLIPping?`` query.
            - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:CLIPping?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - S<x>_CH<x>:CLIPping?
            ```
        """
        return self._clipping

    @property
    def coupling(self) -> SItemChannelCoupling:
        """Return the ``S<x>_CH<x>:COUPling`` command.

        Description:
            - This command sets or queries the input coupling setting for the specified remote scope
              analog channel. The remote scope and the channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``S<x>_CH<x>:COUPling?`` query.
            - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:COUPling?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:COUPling value``
              command.

        SCPI Syntax:
            ```
            - S<x>_CH<x>:COUPling {AC|DC|DCREJ}
            - S<x>_CH<x>:COUPling?
            ```

        Info:
            - ``AC`` sets the specified channel to AC coupling.
            - ``DC`` sets the specified channel to DC coupling.
            - ``DCREJect`` sets DC Reject coupling when probes are attached that support that
              feature.
        """
        return self._coupling

    @property
    def deskew(self) -> SItemChannelDeskew:
        """Return the ``S<x>_CH<x>:DESKew`` command.

        Description:
            - This command sets or queries the horizontal deskew time for the specified channel. The
              remote scope and channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``S<x>_CH<x>:DESKew?`` query.
            - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:DESKew?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:DESKew value`` command.

        SCPI Syntax:
            ```
            - S<x>_CH<x>:DESKew <NR3>
            - S<x>_CH<x>:DESKew?
            ```

        Info:
            - ``<NR3>`` is the deskew time for this channel, ranging from -125 ns to +125 ns with a
              resolution of 40 ps. Out-of-range values are clipped.
        """
        return self._deskew

    @property
    def label(self) -> SItemChannelLabel:
        """Return the ``S<x>_CH<x>:LABel`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``S<x>_CH<x>:LABel?`` query.
            - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:LABel?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.color``: The ``S<x>_CH<x>:LABel:COLor`` command.
            - ``.font``: The ``S<x>_CH<x>:LABel:FONT`` command tree.
            - ``.name``: The ``S<x>_CH<x>:LABel:NAMe`` command.
            - ``.xpos``: The ``S<x>_CH<x>:LABel:XPOS`` command.
            - ``.ypos``: The ``S<x>_CH<x>:LABel:YPOS`` command.
        """
        return self._label

    @property
    def offset(self) -> SItemChannelOffset:
        """Return the ``S<x>_CH<x>:OFFSet`` command.

        Description:
            - This command sets or queries the vertical offset for the specified remote scope analog
              channel. The remote scope and channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``S<x>_CH<x>:OFFSet?`` query.
            - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>:OFFSet?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:OFFSet value`` command.

        SCPI Syntax:
            ```
            - S<x>_CH<x>:OFFSet <NR3>
            - S<x>_CH<x>:OFFSet?
            ```

        Info:
            - ``<NR3>`` is the offset value for the specified remote scope analog channel.
        """
        return self._offset

    @property
    def position(self) -> SItemChannelPosition:
        """Return the ``S<x>_CH<x>:POSition`` command.

        Description:
            - This command sets or queries the vertical position for the specified analog channel.
              The remote scope and channel is specified by x.

        Usage:
            - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:POSition value``
              command.

        SCPI Syntax:
            ```
            - S<x>_CH<x>:POSition <NR1>
            ```

        Info:
            - ``<NR1>`` is the vertical position for the specified analog channel.
        """
        return self._position

    @property
    def scale(self) -> SItemChannelScale:
        """Return the ``S<x>_CH<x>:SCAle`` command.

        Description:
            - This command sets or returns the vertical scale for the specified remote scope analog
              channel. The remote scope and the channel is specified by x.

        Usage:
            - Using the ``.write(value)`` method will send the ``S<x>_CH<x>:SCAle value`` command.

        SCPI Syntax:
            ```
            - S<x>_CH<x>:SCAle <NR3>
            ```

        Info:
            - ``<NR3>`` is the vertical scale for the specified analog channel.
        """
        return self._scale


class SItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``S<x>`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``S<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``S<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.ch``: The ``S<x>_CH<x>`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "S<x>") -> None:
        super().__init__(device, cmd_syntax)
        self._ch: Dict[int, SItemChannel] = DefaultDictPassKeyToFactory(
            lambda x: SItemChannel(device, f"{self._cmd_syntax}_CH{x}")
        )

    @property
    def ch(self) -> Dict[int, SItemChannel]:
        """Return the ``S<x>_CH<x>`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``S<x>_CH<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``S<x>_CH<x>?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.bandwidth``: The ``S<x>_CH<x>:BANdwidth`` command tree.
            - ``.clipping``: The ``S<x>_CH<x>:CLIPping`` command.
            - ``.coupling``: The ``S<x>_CH<x>:COUPling`` command.
            - ``.deskew``: The ``S<x>_CH<x>:DESKew`` command.
            - ``.label``: The ``S<x>_CH<x>:LABel`` command tree.
            - ``.offset``: The ``S<x>_CH<x>:OFFSet`` command.
            - ``.position``: The ``S<x>_CH<x>:POSition`` command.
            - ``.scale``: The ``S<x>_CH<x>:SCAle`` command.
        """
        return self._ch



"""The wfmpre commands module.

These commands are used in the following models:
DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - WFMPre:NR_FR?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class WfmpreNrFr(SCPICmdRead):
    """The ``WFMPre:NR_FR`` command.

    Description:
        - This query-only command returns the number of frames for the waveform transmitted in
          response to a CURVE? query.

    Usage:
        - Using the ``.query()`` method will send the ``WFMPre:NR_FR?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMPre:NR_FR?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMPre:NR_FR?
        ```
    """


class Wfmpre(SCPICmdRead):
    """The ``WFMPre`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``WFMPre?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMPre?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.nr_fr``: The ``WFMPre:NR_FR`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "WFMPre") -> None:
        super().__init__(device, cmd_syntax)
        self._nr_fr = WfmpreNrFr(device, f"{self._cmd_syntax}:NR_FR")

    @property
    def nr_fr(self) -> WfmpreNrFr:
        """Return the ``WFMPre:NR_FR`` command.

        Description:
            - This query-only command returns the number of frames for the waveform transmitted in
              response to a CURVE? query.

        Usage:
            - Using the ``.query()`` method will send the ``WFMPre:NR_FR?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMPre:NR_FR?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - WFMPre:NR_FR?
            ```
        """
        return self._nr_fr



"""The curve commands module.

These commands are used in the following models:
MSO2, TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - CURVe?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Curve(SCPICmdRead):
    """The ``CURVe`` command.

    Description:
        - This command transfers waveform data from the instrument. Each waveform that is
          transferred has an associated waveform preamble that contains information such as data
          format and scale. The ``CURVe?`` query transfers data from the instrument. The data source
          is specified by the ``DATA:SOURCE`` command. The first and last data points are specified
          by the ``DATA:START`` and ``DATA:STOP`` commands. For MATH sources, only 8-byte double
          precision floating point data is returned in ``:CURVe?`` queries.

    Usage:
        - Using the ``.query()`` method will send the ``CURVe?`` query.
        - Using the ``.verify(value)`` method will send the ``CURVe?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CURVe?
        ```
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "CURVe") -> None:
        super().__init__(device, cmd_syntax)



"""The mathvar commands module.

These commands are used in the following models:
DPO2K, DPO2KB, DPO4K, DPO4KB, DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC,
DSA70KC, DSA70KD, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2K, MSO2KB, MSO4K, MSO4KB, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - MATHVAR:VAR<x> <NR3>
    - MATHVAR:VAR<x>?
    - MATHVAR?
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING

from ..helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class MathvarVarItem(ValidatedDynamicNumberCmd, SCPICmdWrite, SCPICmdRead):
    r"""The ``MATHVAR:VAR<x>`` command.

    Description:
        - This command specifies one of two different numerical values you can use within math
          expressions. These values can range from -10.0e-18 to 1.0e+15; the default values are 0.0.
          <x> specifies the location, 1 or 2, in which you can store values. Stored math variables
          can be referenced within math expressions as VAR1 and VAR2. For example, the following
          command defines MATH1 as the product of Channel 1 and math variable 1: ``MATH1:DEFINE``
          'CH1 \* VAR1'.

    Usage:
        - Using the ``.query()`` method will send the ``MATHVAR:VAR<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``MATHVAR:VAR<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MATHVAR:VAR<x> value`` command.

    SCPI Syntax:
        ```
        - MATHVAR:VAR<x> <NR3>
        - MATHVAR:VAR<x>?
        ```

    Info:
        - ``<NR3>`` is a floating point number that specifies the numerical value to be stored in
          location x <1 through 2>.
    """


class Mathvar(SCPICmdRead):
    """The ``MATHVAR`` command.

    Description:
        - Queries both numerical values you can use within math expressions.

    Usage:
        - Using the ``.query()`` method will send the ``MATHVAR?`` query.
        - Using the ``.verify(value)`` method will send the ``MATHVAR?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MATHVAR?
        ```

    Properties:
        - ``.var``: The ``MATHVAR:VAR<x>`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "MATHVAR") -> None:
        super().__init__(device, cmd_syntax)
        self._var: Dict[int, MathvarVarItem] = DefaultDictPassKeyToFactory(
            lambda x: MathvarVarItem(device, f"{self._cmd_syntax}:VAR{x}")
        )

    @property
    def var(self) -> Dict[int, MathvarVarItem]:
        r"""Return the ``MATHVAR:VAR<x>`` command.

        Description:
            - This command specifies one of two different numerical values you can use within math
              expressions. These values can range from -10.0e-18 to 1.0e+15; the default values are
              0.0. <x> specifies the location, 1 or 2, in which you can store values. Stored math
              variables can be referenced within math expressions as VAR1 and VAR2. For example, the
              following command defines MATH1 as the product of Channel 1 and math variable 1:
              ``MATH1:DEFINE`` 'CH1 \* VAR1'.

        Usage:
            - Using the ``.query()`` method will send the ``MATHVAR:VAR<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``MATHVAR:VAR<x>?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MATHVAR:VAR<x> value`` command.

        SCPI Syntax:
            ```
            - MATHVAR:VAR<x> <NR3>
            - MATHVAR:VAR<x>?
            ```

        Info:
            - ``<NR3>`` is a floating point number that specifies the numerical value to be stored
              in location x <1 through 2>.
        """
        return self._var



# pylint: disable=line-too-long
"""The visual commands module.

These commands are used in the following models:
DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - VISual:AREA<x>:DISplay {ON|OFF}
    - VISual:AREA<x>:DISplay?
    - VISual:AREA<x>:FLIP {HORIZONTAL|VERTICAL}
    - VISual:AREA<x>:HEIGHT <nr3>
    - VISual:AREA<x>:HEIGHT?
    - VISual:AREA<x>:LABel <string>
    - VISual:AREA<x>:LABel?
    - VISual:AREA<x>:OPERATION<x> {IN|OUT}
    - VISual:AREA<x>:OPERATION<x>?
    - VISual:AREA<x>:ROTAtion <nr3>
    - VISual:AREA<x>:ROTAtion?
    - VISual:AREA<x>:SHAPE {HEXAgon|RECTangle|TRAPezoid|TRIAngle}
    - VISual:AREA<x>:SHAPE?
    - VISual:AREA<x>:SHOWLOGic {ON|OFF}
    - VISual:AREA<x>:SHOWLOGic?
    - VISual:AREA<x>:SOURCE<x> {WFMCH<x>}
    - VISual:AREA<x>:SOURCE<x>?
    - VISual:AREA<x>:VERTICES {<NR3>,<NR3>,<NR3>,<NR3>,<NR3>,<NR3> [,<NR3>,<NR3>,<NR3>,<NR3>[,<NR3>,<NR3>]]}
    - VISual:AREA<x>:VERTICES?
    - VISual:AREA<x>:WIDTH <nr3>
    - VISual:AREA<x>:WIDTH?
    - VISual:AREA<x>:XPOSition <nr3>
    - VISual:AREA<x>:XPOSition?
    - VISual:AREA<x>:YPOSition <nr3>
    - VISual:AREA<x>:YPOSition?
    - VISual:AREA<x>?
    - VISual:AREACOLOr {DEFAULT|INHERIT}
    - VISual:AREACOLOr?
    - VISual:ASPECTratio {ON|OFF|<NR1>}
    - VISual:ASPECTratio?
    - VISual:DELETEAREA <1-8>
    - VISual:ENAble {ON|OFF}
    - VISual:ENAble?
    - VISual:FILE:RECALL <string>
    - VISual:FILE:SAVE <string>
    - VISual?
    ```
"""  # noqa: E501

from typing import Dict, Optional, TYPE_CHECKING

from ..helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class VisualFileSave(SCPICmdWrite):
    """The ``VISual:FILE:SAVE`` command.

    Description:
        - Saves only the Visual Trigger Parameters, to a file that you name in the <string>. The
          file is stored in the form of programmer interface commands, which are human readable.

    Usage:
        - Using the ``.write(value)`` method will send the ``VISual:FILE:SAVE value`` command.

    SCPI Syntax:
        ```
        - VISual:FILE:SAVE <string>
        ```

    Info:
        - ``<string>`` is the complete path and file name where you want to store the visual trigger
          parameters. This can be a local or network drive.
    """


class VisualFileRecall(SCPICmdWrite):
    """The ``VISual:FILE:RECALL`` command.

    Description:
        - Recalls a file of Visual Trigger Parameters, updating the database appropriately. The file
          is stored in the form of programmer interface commands, which are human readable.

    Usage:
        - Using the ``.write(value)`` method will send the ``VISual:FILE:RECALL value`` command.

    SCPI Syntax:
        ```
        - VISual:FILE:RECALL <string>
        ```
    """


class VisualFile(SCPICmdRead):
    """The ``VISual:FILE`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``VISual:FILE?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:FILE?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.recall``: The ``VISual:FILE:RECALL`` command.
        - ``.save``: The ``VISual:FILE:SAVE`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._recall = VisualFileRecall(device, f"{self._cmd_syntax}:RECALL")
        self._save = VisualFileSave(device, f"{self._cmd_syntax}:SAVE")

    @property
    def recall(self) -> VisualFileRecall:
        """Return the ``VISual:FILE:RECALL`` command.

        Description:
            - Recalls a file of Visual Trigger Parameters, updating the database appropriately. The
              file is stored in the form of programmer interface commands, which are human readable.

        Usage:
            - Using the ``.write(value)`` method will send the ``VISual:FILE:RECALL value`` command.

        SCPI Syntax:
            ```
            - VISual:FILE:RECALL <string>
            ```
        """
        return self._recall

    @property
    def save(self) -> VisualFileSave:
        """Return the ``VISual:FILE:SAVE`` command.

        Description:
            - Saves only the Visual Trigger Parameters, to a file that you name in the <string>. The
              file is stored in the form of programmer interface commands, which are human readable.

        Usage:
            - Using the ``.write(value)`` method will send the ``VISual:FILE:SAVE value`` command.

        SCPI Syntax:
            ```
            - VISual:FILE:SAVE <string>
            ```

        Info:
            - ``<string>`` is the complete path and file name where you want to store the visual
              trigger parameters. This can be a local or network drive.
        """
        return self._save


class VisualEnable(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:ENAble`` command.

    Description:
        - Enables or disables the visual triggering. Queries the status (on or off) of the visual
          triggering. This does not refer to the option key.

    Usage:
        - Using the ``.query()`` method will send the ``VISual:ENAble?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:ENAble?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:ENAble value`` command.

    SCPI Syntax:
        ```
        - VISual:ENAble {ON|OFF}
        - VISual:ENAble?
        ```

    Info:
        - ``ON`` enables the visual triggering feature.
        - ``OFF`` disables the visual triggering feature.
    """


class VisualDeletearea(SCPICmdWrite):
    """The ``VISual:DELETEAREA`` command.

    Description:
        - This command deletes the specified visual trigger area. The area is specified by x. The
          value of x can range from 1 to 8.

    Usage:
        - Using the ``.write(value)`` method will send the ``VISual:DELETEAREA value`` command.

    SCPI Syntax:
        ```
        - VISual:DELETEAREA <1-8>
        ```
    """


class VisualAspectratio(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:ASPECTratio`` command.

    Description:
        - This command sets or queries the aspect ratio setting of the visual trigger system.

    Usage:
        - Using the ``.query()`` method will send the ``VISual:ASPECTratio?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:ASPECTratio?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:ASPECTratio value`` command.

    SCPI Syntax:
        ```
        - VISual:ASPECTratio {ON|OFF|<NR1>}
        - VISual:ASPECTratio?
        ```

    Info:
        - ``<NR1>`` = 0 disables the function; any other value enables it.
        - ``OFF`` disables the function.
        - ``ON`` enables keeping the aspect ratio constant.
    """


class VisualAreacolor(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREACOLOr`` command.

    Description:
        - This command sets or queries the colors used by visual trigger areas.

    Usage:
        - Using the ``.query()`` method will send the ``VISual:AREACOLOr?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREACOLOr?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREACOLOr value`` command.

    SCPI Syntax:
        ```
        - VISual:AREACOLOr {DEFAULT|INHERIT}
        - VISual:AREACOLOr?
        ```

    Info:
        - ``DEFAULT`` sets visual trigger areas to use the default blue color.
        - ``INHERIT`` sets visual trigger areas to inherit the color of the channel.
    """


class VisualAreaItemYposition(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:YPOSition`` command.

    Description:
        - This command sets or queries vertical position of the specified visual trigger area. The
          area is specified by x. The value of x can range from 1 to 8.

    Usage:
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:YPOSition?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:YPOSition?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:YPOSition value``
          command.

    SCPI Syntax:
        ```
        - VISual:AREA<x>:YPOSition <nr3>
        - VISual:AREA<x>:YPOSition?
        ```

    Info:
        - ``<NR3>`` specifies the vertical position of the center of the visual trigger area.
    """


class VisualAreaItemXposition(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:XPOSition`` command.

    Description:
        - This command sets or queries horizontal position of specified visual trigger area. The
          area is specified by x. The value of x can range from 1 to 8.

    Usage:
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:XPOSition?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:XPOSition?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:XPOSition value``
          command.

    SCPI Syntax:
        ```
        - VISual:AREA<x>:XPOSition <nr3>
        - VISual:AREA<x>:XPOSition?
        ```

    Info:
        - ``<NR3>`` specifies the horizontal position of the center of the visual trigger area.
    """


class VisualAreaItemWidth(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:WIDTH`` command.

    Description:
        - This command sets or queries the width of the specified visual trigger area. The area is
          specified by x. The value of x can range from 1 to 8.

    Usage:
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:WIDTH?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:WIDTH?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:WIDTH value`` command.

    SCPI Syntax:
        ```
        - VISual:AREA<x>:WIDTH <nr3>
        - VISual:AREA<x>:WIDTH?
        ```

    Info:
        - ``<NR3>`` specifies the width of the visual trigger area.
    """


class VisualAreaItemVertices(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:VERTICES`` command.

    Description:
        - Sets or queries the polygon vertex x and y coordinate values for an area. You must set
          vertex values in pairs; you can set 3, 4, or 6 pairs (no pentagons are allowed).

    Usage:
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:VERTICES?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:VERTICES?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:VERTICES value``
          command.

    SCPI Syntax:
        ```
        - VISual:AREA<x>:VERTICES {<NR3>,<NR3>,<NR3>,<NR3>,<NR3>,<NR3> [,<NR3>,<NR3>,<NR3>,<NR3>[,<NR3>,<NR3>]]}
        - VISual:AREA<x>:VERTICES?
        ```
    """  # noqa: E501


class VisualAreaItemSourceItem(ValidatedDynamicNumberCmd, SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:SOURCE<x>`` command.

    Description:
        - Selects or queries the signal source for the area <x>. The source can only be an analog
          channel. You can enter the command as ``VISUAL:AREA<x>:SOURCE`` or
          ``VISUAL:AREA<x>:SOURCE1``.

    Usage:
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:SOURCE<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:SOURCE<x>?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:SOURCE<x> value``
          command.

    SCPI Syntax:
        ```
        - VISual:AREA<x>:SOURCE<x> {WFMCH<x>}
        - VISual:AREA<x>:SOURCE<x>?
        ```

    Info:
        - ``WFMCH1`` indicates that the analog channel 1 waveform is used as the signal source for
          the area <x>. The other three channels have the same syntax.
    """


class VisualAreaItemShowlogic(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:SHOWLOGic`` command.

    Description:
        - Causes area Source and Operation indicators to be hidden or displayed on screen. Queries
          the status of this feature. All areas are turned on and off together. The <x> is reserved
          for future use.

    Usage:
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:SHOWLOGic?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:SHOWLOGic?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:SHOWLOGic value``
          command.

    SCPI Syntax:
        ```
        - VISual:AREA<x>:SHOWLOGic {ON|OFF}
        - VISual:AREA<x>:SHOWLOGic?
        ```

    Info:
        - ``ON`` shows the definition (source, operation, and area label) of each area within the
          graphical display of that area.
        - ``OFF`` removes the definitions from the display.
    """


class VisualAreaItemShape(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:SHAPE`` command.

    Description:
        - Selects the initial shape of area <x>.

    Usage:
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:SHAPE?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:SHAPE?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:SHAPE value`` command.

    SCPI Syntax:
        ```
        - VISual:AREA<x>:SHAPE {HEXAgon|RECTangle|TRAPezoid|TRIAngle}
        - VISual:AREA<x>:SHAPE?
        ```

    Info:
        - ``HEXAGON`` sets the initial shape of the selected area to a hexagon.
        - ``RECTANGLE`` sets the initial shape of the selected area to a rectangle.
        - ``TRAPEZOID`` sets the initial shape of the selected area to a trapezoid.
        - ``TRIANGLE`` sets the initial shape of the selected area to a triangle.
    """


class VisualAreaItemRotation(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:ROTAtion`` command.

    Description:
        - This command sets or queries the rotation angle of the specified visual trigger area. The
          area is specified by x. The value of x can range from 1 to 8.

    Usage:
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:ROTAtion?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:ROTAtion?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:ROTAtion value``
          command.

    SCPI Syntax:
        ```
        - VISual:AREA<x>:ROTAtion <nr3>
        - VISual:AREA<x>:ROTAtion?
        ```

    Info:
        - ``<NR3>`` specifies the rotation angle of the specified visual trigger area.
    """


class VisualAreaItemOperationItem(ValidatedDynamicNumberCmd, SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:OPERATION<x>`` command.

    Description:
        - Sets or queries the operation of the area specified by <x>. You can enter the command as
          ``VISUAL:AREA<x>:OPERATION`` or ``VISUAL:AREA<x>:OPERATION1``.

    Usage:
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:OPERATION<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:OPERATION<x>?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:OPERATION<x> value``
          command.

    SCPI Syntax:
        ```
        - VISual:AREA<x>:OPERATION<x> {IN|OUT}
        - VISual:AREA<x>:OPERATION<x>?
        ```

    Info:
        - ``IN`` specifies that the selected source must pass through the specified area.
        - ``OUT`` specifies that the selected source must NOT pass through the specified area.
    """


class VisualAreaItemLabel(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:LABel`` command.

    Description:
        - This command sets or queries the label of the specified visual trigger area. The area is
          specified by x. The value of x can range from 1 to 8.

    Usage:
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:LABel?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:LABel?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:LABel value`` command.

    SCPI Syntax:
        ```
        - VISual:AREA<x>:LABel <string>
        - VISual:AREA<x>:LABel?
        ```

    Info:
        - ``<string>`` specifies the label for the area.
    """


class VisualAreaItemHeight(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:HEIGHT`` command.

    Description:
        - This command sets or queries the height of the specified visual trigger area. The area is
          specified by x. The value of x can range from 1 to 8.

    Usage:
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:HEIGHT?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:HEIGHT?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:HEIGHT value`` command.

    SCPI Syntax:
        ```
        - VISual:AREA<x>:HEIGHT <nr3>
        - VISual:AREA<x>:HEIGHT?
        ```

    Info:
        - ``<NR3>`` specifies the height of the specified visual trigger area.
    """


class VisualAreaItemFlip(SCPICmdWrite):
    """The ``VISual:AREA<x>:FLIP`` command.

    Description:
        - This command flips the specified visual trigger area. The area is specified by x. The
          value of x can range from 1 to 8.

    Usage:
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:FLIP value`` command.

    SCPI Syntax:
        ```
        - VISual:AREA<x>:FLIP {HORIZONTAL|VERTICAL}
        ```

    Info:
        - ``HORIZONTAL`` specifies to flip the area in the horizontal direction.
        - ``VERTICAL`` specifies to flip the area in the vertical direction.
    """


class VisualAreaItemDisplay(SCPICmdWrite, SCPICmdRead):
    """The ``VISual:AREA<x>:DISplay`` command.

    Description:
        - Causes the areas to be hidden or displayed on the oscilloscope screen. Queries the status
          of the area display. All areas are set On or Off together. The <x> in the command is
          reserved for future use.

    Usage:
        - Using the ``.query()`` method will send the ``VISual:AREA<x>:DISplay?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:DISplay?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:DISplay value`` command.

    SCPI Syntax:
        ```
        - VISual:AREA<x>:DISplay {ON|OFF}
        - VISual:AREA<x>:DISplay?
        ```

    Info:
        - ``ON`` causes the areas to appear on screen.
        - ``Off`` hides the areas.
    """


#  pylint: disable=too-many-instance-attributes
class VisualAreaItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``VISual:AREA<x>`` command.

    Description:
        - This query-only command returns the settings for the specified visual trigger area. The
          area is specified by x. The value of x can range from 1 through 8.

    Usage:
        - Using the ``.query()`` method will send the ``VISual:AREA<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - VISual:AREA<x>?
        ```

    Properties:
        - ``.display``: The ``VISual:AREA<x>:DISplay`` command.
        - ``.flip``: The ``VISual:AREA<x>:FLIP`` command.
        - ``.height``: The ``VISual:AREA<x>:HEIGHT`` command.
        - ``.label``: The ``VISual:AREA<x>:LABel`` command.
        - ``.operation``: The ``VISual:AREA<x>:OPERATION<x>`` command.
        - ``.rotation``: The ``VISual:AREA<x>:ROTAtion`` command.
        - ``.shape``: The ``VISual:AREA<x>:SHAPE`` command.
        - ``.showlogic``: The ``VISual:AREA<x>:SHOWLOGic`` command.
        - ``.source``: The ``VISual:AREA<x>:SOURCE<x>`` command.
        - ``.vertices``: The ``VISual:AREA<x>:VERTICES`` command.
        - ``.width``: The ``VISual:AREA<x>:WIDTH`` command.
        - ``.xposition``: The ``VISual:AREA<x>:XPOSition`` command.
        - ``.yposition``: The ``VISual:AREA<x>:YPOSition`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._display = VisualAreaItemDisplay(device, f"{self._cmd_syntax}:DISplay")
        self._flip = VisualAreaItemFlip(device, f"{self._cmd_syntax}:FLIP")
        self._height = VisualAreaItemHeight(device, f"{self._cmd_syntax}:HEIGHT")
        self._label = VisualAreaItemLabel(device, f"{self._cmd_syntax}:LABel")
        self._operation: Dict[int, VisualAreaItemOperationItem] = DefaultDictPassKeyToFactory(
            lambda x: VisualAreaItemOperationItem(device, f"{self._cmd_syntax}:OPERATION{x}")
        )
        self._rotation = VisualAreaItemRotation(device, f"{self._cmd_syntax}:ROTAtion")
        self._shape = VisualAreaItemShape(device, f"{self._cmd_syntax}:SHAPE")
        self._showlogic = VisualAreaItemShowlogic(device, f"{self._cmd_syntax}:SHOWLOGic")
        self._source: Dict[int, VisualAreaItemSourceItem] = DefaultDictPassKeyToFactory(
            lambda x: VisualAreaItemSourceItem(device, f"{self._cmd_syntax}:SOURCE{x}")
        )
        self._vertices = VisualAreaItemVertices(device, f"{self._cmd_syntax}:VERTICES")
        self._width = VisualAreaItemWidth(device, f"{self._cmd_syntax}:WIDTH")
        self._xposition = VisualAreaItemXposition(device, f"{self._cmd_syntax}:XPOSition")
        self._yposition = VisualAreaItemYposition(device, f"{self._cmd_syntax}:YPOSition")

    @property
    def display(self) -> VisualAreaItemDisplay:
        """Return the ``VISual:AREA<x>:DISplay`` command.

        Description:
            - Causes the areas to be hidden or displayed on the oscilloscope screen. Queries the
              status of the area display. All areas are set On or Off together. The <x> in the
              command is reserved for future use.

        Usage:
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:DISplay?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:DISplay?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:DISplay value``
              command.

        SCPI Syntax:
            ```
            - VISual:AREA<x>:DISplay {ON|OFF}
            - VISual:AREA<x>:DISplay?
            ```

        Info:
            - ``ON`` causes the areas to appear on screen.
            - ``Off`` hides the areas.
        """
        return self._display

    @property
    def flip(self) -> VisualAreaItemFlip:
        """Return the ``VISual:AREA<x>:FLIP`` command.

        Description:
            - This command flips the specified visual trigger area. The area is specified by x. The
              value of x can range from 1 to 8.

        Usage:
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:FLIP value``
              command.

        SCPI Syntax:
            ```
            - VISual:AREA<x>:FLIP {HORIZONTAL|VERTICAL}
            ```

        Info:
            - ``HORIZONTAL`` specifies to flip the area in the horizontal direction.
            - ``VERTICAL`` specifies to flip the area in the vertical direction.
        """
        return self._flip

    @property
    def height(self) -> VisualAreaItemHeight:
        """Return the ``VISual:AREA<x>:HEIGHT`` command.

        Description:
            - This command sets or queries the height of the specified visual trigger area. The area
              is specified by x. The value of x can range from 1 to 8.

        Usage:
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:HEIGHT?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:HEIGHT?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:HEIGHT value``
              command.

        SCPI Syntax:
            ```
            - VISual:AREA<x>:HEIGHT <nr3>
            - VISual:AREA<x>:HEIGHT?
            ```

        Info:
            - ``<NR3>`` specifies the height of the specified visual trigger area.
        """
        return self._height

    @property
    def label(self) -> VisualAreaItemLabel:
        """Return the ``VISual:AREA<x>:LABel`` command.

        Description:
            - This command sets or queries the label of the specified visual trigger area. The area
              is specified by x. The value of x can range from 1 to 8.

        Usage:
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:LABel?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:LABel?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:LABel value``
              command.

        SCPI Syntax:
            ```
            - VISual:AREA<x>:LABel <string>
            - VISual:AREA<x>:LABel?
            ```

        Info:
            - ``<string>`` specifies the label for the area.
        """
        return self._label

    @property
    def operation(self) -> Dict[int, VisualAreaItemOperationItem]:
        """Return the ``VISual:AREA<x>:OPERATION<x>`` command.

        Description:
            - Sets or queries the operation of the area specified by <x>. You can enter the command
              as ``VISUAL:AREA<x>:OPERATION`` or ``VISUAL:AREA<x>:OPERATION1``.

        Usage:
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:OPERATION<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:OPERATION<x>?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:OPERATION<x> value``
              command.

        SCPI Syntax:
            ```
            - VISual:AREA<x>:OPERATION<x> {IN|OUT}
            - VISual:AREA<x>:OPERATION<x>?
            ```

        Info:
            - ``IN`` specifies that the selected source must pass through the specified area.
            - ``OUT`` specifies that the selected source must NOT pass through the specified area.
        """
        return self._operation

    @property
    def rotation(self) -> VisualAreaItemRotation:
        """Return the ``VISual:AREA<x>:ROTAtion`` command.

        Description:
            - This command sets or queries the rotation angle of the specified visual trigger area.
              The area is specified by x. The value of x can range from 1 to 8.

        Usage:
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:ROTAtion?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:ROTAtion?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:ROTAtion value``
              command.

        SCPI Syntax:
            ```
            - VISual:AREA<x>:ROTAtion <nr3>
            - VISual:AREA<x>:ROTAtion?
            ```

        Info:
            - ``<NR3>`` specifies the rotation angle of the specified visual trigger area.
        """
        return self._rotation

    @property
    def shape(self) -> VisualAreaItemShape:
        """Return the ``VISual:AREA<x>:SHAPE`` command.

        Description:
            - Selects the initial shape of area <x>.

        Usage:
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:SHAPE?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:SHAPE?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:SHAPE value``
              command.

        SCPI Syntax:
            ```
            - VISual:AREA<x>:SHAPE {HEXAgon|RECTangle|TRAPezoid|TRIAngle}
            - VISual:AREA<x>:SHAPE?
            ```

        Info:
            - ``HEXAGON`` sets the initial shape of the selected area to a hexagon.
            - ``RECTANGLE`` sets the initial shape of the selected area to a rectangle.
            - ``TRAPEZOID`` sets the initial shape of the selected area to a trapezoid.
            - ``TRIANGLE`` sets the initial shape of the selected area to a triangle.
        """
        return self._shape

    @property
    def showlogic(self) -> VisualAreaItemShowlogic:
        """Return the ``VISual:AREA<x>:SHOWLOGic`` command.

        Description:
            - Causes area Source and Operation indicators to be hidden or displayed on screen.
              Queries the status of this feature. All areas are turned on and off together. The <x>
              is reserved for future use.

        Usage:
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:SHOWLOGic?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:SHOWLOGic?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:SHOWLOGic value``
              command.

        SCPI Syntax:
            ```
            - VISual:AREA<x>:SHOWLOGic {ON|OFF}
            - VISual:AREA<x>:SHOWLOGic?
            ```

        Info:
            - ``ON`` shows the definition (source, operation, and area label) of each area within
              the graphical display of that area.
            - ``OFF`` removes the definitions from the display.
        """
        return self._showlogic

    @property
    def source(self) -> Dict[int, VisualAreaItemSourceItem]:
        """Return the ``VISual:AREA<x>:SOURCE<x>`` command.

        Description:
            - Selects or queries the signal source for the area <x>. The source can only be an
              analog channel. You can enter the command as ``VISUAL:AREA<x>:SOURCE`` or
              ``VISUAL:AREA<x>:SOURCE1``.

        Usage:
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:SOURCE<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:SOURCE<x>?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:SOURCE<x> value``
              command.

        SCPI Syntax:
            ```
            - VISual:AREA<x>:SOURCE<x> {WFMCH<x>}
            - VISual:AREA<x>:SOURCE<x>?
            ```

        Info:
            - ``WFMCH1`` indicates that the analog channel 1 waveform is used as the signal source
              for the area <x>. The other three channels have the same syntax.
        """
        return self._source

    @property
    def vertices(self) -> VisualAreaItemVertices:
        """Return the ``VISual:AREA<x>:VERTICES`` command.

        Description:
            - Sets or queries the polygon vertex x and y coordinate values for an area. You must set
              vertex values in pairs; you can set 3, 4, or 6 pairs (no pentagons are allowed).

        Usage:
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:VERTICES?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:VERTICES?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:VERTICES value``
              command.

        SCPI Syntax:
            ```
            - VISual:AREA<x>:VERTICES {<NR3>,<NR3>,<NR3>,<NR3>,<NR3>,<NR3> [,<NR3>,<NR3>,<NR3>,<NR3>[,<NR3>,<NR3>]]}
            - VISual:AREA<x>:VERTICES?
            ```
        """  # noqa: E501
        return self._vertices

    @property
    def width(self) -> VisualAreaItemWidth:
        """Return the ``VISual:AREA<x>:WIDTH`` command.

        Description:
            - This command sets or queries the width of the specified visual trigger area. The area
              is specified by x. The value of x can range from 1 to 8.

        Usage:
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:WIDTH?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:WIDTH?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:WIDTH value``
              command.

        SCPI Syntax:
            ```
            - VISual:AREA<x>:WIDTH <nr3>
            - VISual:AREA<x>:WIDTH?
            ```

        Info:
            - ``<NR3>`` specifies the width of the visual trigger area.
        """
        return self._width

    @property
    def xposition(self) -> VisualAreaItemXposition:
        """Return the ``VISual:AREA<x>:XPOSition`` command.

        Description:
            - This command sets or queries horizontal position of specified visual trigger area. The
              area is specified by x. The value of x can range from 1 to 8.

        Usage:
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:XPOSition?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:XPOSition?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:XPOSition value``
              command.

        SCPI Syntax:
            ```
            - VISual:AREA<x>:XPOSition <nr3>
            - VISual:AREA<x>:XPOSition?
            ```

        Info:
            - ``<NR3>`` specifies the horizontal position of the center of the visual trigger area.
        """
        return self._xposition

    @property
    def yposition(self) -> VisualAreaItemYposition:
        """Return the ``VISual:AREA<x>:YPOSition`` command.

        Description:
            - This command sets or queries vertical position of the specified visual trigger area.
              The area is specified by x. The value of x can range from 1 to 8.

        Usage:
            - Using the ``.query()`` method will send the ``VISual:AREA<x>:YPOSition?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>:YPOSition?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREA<x>:YPOSition value``
              command.

        SCPI Syntax:
            ```
            - VISual:AREA<x>:YPOSition <nr3>
            - VISual:AREA<x>:YPOSition?
            ```

        Info:
            - ``<NR3>`` specifies the vertical position of the center of the visual trigger area.
        """
        return self._yposition


class Visual(SCPICmdRead):
    """The ``VISual`` command.

    Description:
        - This query-only command returns the settings for each visual trigger area.

    Usage:
        - Using the ``.query()`` method will send the ``VISual?`` query.
        - Using the ``.verify(value)`` method will send the ``VISual?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - VISual?
        ```

    Properties:
        - ``.area``: The ``VISual:AREA<x>`` command.
        - ``.areacolor``: The ``VISual:AREACOLOr`` command.
        - ``.aspectratio``: The ``VISual:ASPECTratio`` command.
        - ``.deletearea``: The ``VISual:DELETEAREA`` command.
        - ``.enable``: The ``VISual:ENAble`` command.
        - ``.file``: The ``VISual:FILE`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "VISual") -> None:
        super().__init__(device, cmd_syntax)
        self._area: Dict[int, VisualAreaItem] = DefaultDictPassKeyToFactory(
            lambda x: VisualAreaItem(device, f"{self._cmd_syntax}:AREA{x}")
        )
        self._areacolor = VisualAreacolor(device, f"{self._cmd_syntax}:AREACOLOr")
        self._aspectratio = VisualAspectratio(device, f"{self._cmd_syntax}:ASPECTratio")
        self._deletearea = VisualDeletearea(device, f"{self._cmd_syntax}:DELETEAREA")
        self._enable = VisualEnable(device, f"{self._cmd_syntax}:ENAble")
        self._file = VisualFile(device, f"{self._cmd_syntax}:FILE")

    @property
    def area(self) -> Dict[int, VisualAreaItem]:
        """Return the ``VISual:AREA<x>`` command.

        Description:
            - This query-only command returns the settings for the specified visual trigger area.
              The area is specified by x. The value of x can range from 1 through 8.

        Usage:
            - Using the ``.query()`` method will send the ``VISual:AREA<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREA<x>?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - VISual:AREA<x>?
            ```

        Sub-properties:
            - ``.display``: The ``VISual:AREA<x>:DISplay`` command.
            - ``.flip``: The ``VISual:AREA<x>:FLIP`` command.
            - ``.height``: The ``VISual:AREA<x>:HEIGHT`` command.
            - ``.label``: The ``VISual:AREA<x>:LABel`` command.
            - ``.operation``: The ``VISual:AREA<x>:OPERATION<x>`` command.
            - ``.rotation``: The ``VISual:AREA<x>:ROTAtion`` command.
            - ``.shape``: The ``VISual:AREA<x>:SHAPE`` command.
            - ``.showlogic``: The ``VISual:AREA<x>:SHOWLOGic`` command.
            - ``.source``: The ``VISual:AREA<x>:SOURCE<x>`` command.
            - ``.vertices``: The ``VISual:AREA<x>:VERTICES`` command.
            - ``.width``: The ``VISual:AREA<x>:WIDTH`` command.
            - ``.xposition``: The ``VISual:AREA<x>:XPOSition`` command.
            - ``.yposition``: The ``VISual:AREA<x>:YPOSition`` command.
        """
        return self._area

    @property
    def areacolor(self) -> VisualAreacolor:
        """Return the ``VISual:AREACOLOr`` command.

        Description:
            - This command sets or queries the colors used by visual trigger areas.

        Usage:
            - Using the ``.query()`` method will send the ``VISual:AREACOLOr?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:AREACOLOr?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:AREACOLOr value`` command.

        SCPI Syntax:
            ```
            - VISual:AREACOLOr {DEFAULT|INHERIT}
            - VISual:AREACOLOr?
            ```

        Info:
            - ``DEFAULT`` sets visual trigger areas to use the default blue color.
            - ``INHERIT`` sets visual trigger areas to inherit the color of the channel.
        """
        return self._areacolor

    @property
    def aspectratio(self) -> VisualAspectratio:
        """Return the ``VISual:ASPECTratio`` command.

        Description:
            - This command sets or queries the aspect ratio setting of the visual trigger system.

        Usage:
            - Using the ``.query()`` method will send the ``VISual:ASPECTratio?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:ASPECTratio?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:ASPECTratio value`` command.

        SCPI Syntax:
            ```
            - VISual:ASPECTratio {ON|OFF|<NR1>}
            - VISual:ASPECTratio?
            ```

        Info:
            - ``<NR1>`` = 0 disables the function; any other value enables it.
            - ``OFF`` disables the function.
            - ``ON`` enables keeping the aspect ratio constant.
        """
        return self._aspectratio

    @property
    def deletearea(self) -> VisualDeletearea:
        """Return the ``VISual:DELETEAREA`` command.

        Description:
            - This command deletes the specified visual trigger area. The area is specified by x.
              The value of x can range from 1 to 8.

        Usage:
            - Using the ``.write(value)`` method will send the ``VISual:DELETEAREA value`` command.

        SCPI Syntax:
            ```
            - VISual:DELETEAREA <1-8>
            ```
        """
        return self._deletearea

    @property
    def enable(self) -> VisualEnable:
        """Return the ``VISual:ENAble`` command.

        Description:
            - Enables or disables the visual triggering. Queries the status (on or off) of the
              visual triggering. This does not refer to the option key.

        Usage:
            - Using the ``.query()`` method will send the ``VISual:ENAble?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:ENAble?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``VISual:ENAble value`` command.

        SCPI Syntax:
            ```
            - VISual:ENAble {ON|OFF}
            - VISual:ENAble?
            ```

        Info:
            - ``ON`` enables the visual triggering feature.
            - ``OFF`` disables the visual triggering feature.
        """
        return self._enable

    @property
    def file(self) -> VisualFile:
        """Return the ``VISual:FILE`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``VISual:FILE?`` query.
            - Using the ``.verify(value)`` method will send the ``VISual:FILE?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.recall``: The ``VISual:FILE:RECALL`` command.
            - ``.save``: The ``VISual:FILE:SAVE`` command.
        """
        return self._file



"""The status_and_error commands module.

These commands are used in the following models:
AFG3K, AFG3KB, AFG3KC, AWG5200, AWG5K, AWG5KC, AWG70KA, AWG70KB, AWG7K, AWG7KC, DPO4K, DPO4KB,
DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7AX, DPO7K, DPO7KC, DSA70KC, DSA70KD, LPD6,
MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2, MSO4, MSO4B, MSO4K, MSO4KB, MSO5, MSO5B, MSO5K, MSO5KB,
MSO5LP, MSO6, MSO6B, MSO70KC, MSO70KDX, TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - *OPT?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Opt(SCPICmdRead):
    """The ``*OPT`` command.

    Description:
        - This query-only command returns a comma separated list of installed options as an
          arbitrary ASCII string (no quotes) of the form:
          ``<optionCode>:<optionDescription>``,``<optionCode>:<optionDescription>``... The last
          section of each entry (the text following the last hyphen) indicates the license type. If
          no options are found, NONE is returned.

    Usage:
        - Using the ``.query()`` method will send the ``*OPT?`` query.
        - Using the ``.verify(value)`` method will send the ``*OPT?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - *OPT?
        ```
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*OPT") -> None:
        super().__init__(device, cmd_syntax)



"""DPO7 device driver module."""

from abc import ABC

import pyvisa as visa

from tm_devices.drivers.scopes.tekscope.tekscope import TekScope
from tm_devices.helpers import (
    DeviceConfigEntry,
)
from tm_devices.helpers import (
    ReadOnlyCachedProperty as cached_property,  # noqa: N813
)


class DPO7(TekScope, ABC):
    """DPO7 device driver."""

    def __init__(
        self,
        config_entry: DeviceConfigEntry,
        verbose: bool,
        visa_resource: visa.resources.MessageBasedResource,
        default_visa_timeout: int,
    ) -> None:
        """Create a DPO7 device.

        Args:
            config_entry: A config entry object parsed by the DMConfigParser.
            verbose: A boolean indicating if verbose output should be printed.
            visa_resource: The VISA resource object.
            default_visa_timeout: The default VISA timeout value in milliseconds.
        """
        super().__init__(config_entry, verbose, visa_resource, default_visa_timeout)
        self._reboot_quiet_period = 180

    @cached_property
    def total_channels(self) -> int:
        """Return the total number of channels (all types)."""
        return int(self.model[5])

    ################################################################################################
    # Private Methods
    ################################################################################################
    @staticmethod
    def _get_driver_specific_multipliers() -> float:
        """Return a value to multiply the original Tekscope IAFG frequency by."""
        return 2.0



"""The hostprocessor commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - HOSTProcessor?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Hostprocessor(SCPICmdRead):
    """The ``HOSTProcessor`` command.

    Description:
        - This command queries the host processor that the instrument displays.

    Usage:
        - Using the ``.query()`` method will send the ``HOSTProcessor?`` query.
        - Using the ``.verify(value)`` method will send the ``HOSTProcessor?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - HOSTProcessor?
        ```
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "HOSTProcessor"
    ) -> None:
        super().__init__(device, cmd_syntax)
